(*******************************************************************************
    BkgQuery - Backgroud Query  ver.1.1
    Author: J. Tungli, T-SOFT (c) 2001  mailto:jan.tungli@seznam.cz

    1.support SQL DB connection with ALIAS - params USER NAME=? PASSWORD=?
    2.xBase with BDE Alias connection
    3.xBase direct open (select * from "C:\aaa.dbf" )
      (poperties Database_Alias & Database_DatabaseName must by empty  for xBase direct open)

    example:
      with BkgQuery1 do begin
        Database_Alias:='DBDEMOS';
        Database_DatabaseName:='MY';
        OnExecute:=MyExecute1;  //if not ActiveThread
        Execute;
        ...
        while  ActiveThread do Application.ProcessMessages;  // not ActiveThread
        OnExecute:=MyExecute2;
        Execute;
        ...

      end;
      ...
      procedure Form1.MyExecute1(sender:TObject);
      begin
        with BkgQuery1 do begin
          sql.text:='select * from Country';
          Open;
          Locate('Name','Venez',[loPartialkey]);
        end;
      end;

      procedure Form1.MyExecute2(sender:TObject);
      begin
        BkgQuery1.sql.text:='update to Country(Area=123456)';
        BkgQuery1.Database.StartTransaction;
        try
          BkgQuery1.ExecSQL;
          BkgQuery1.Database.Commit;
        except BkgQuery1.Database.RollBack; end;
      end;

     //Event OnAfterExecute call BkgQuery when thread is ready.

 *******************************************************************************)

unit BkgQuery;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables,stdctrls;

const
   UM_BkgReady=WM_User+191;
type
  TBkgHandle=class(TStaticText)  //only for handle of BkgQuery
  private
    fMaster:TComponent;
  protected
    procedure UM_BkgReady(var AMessage : TMessage); message UM_BkgReady;
    { Protected declarations }
  public
  published
  end;

  TBkgQuery = class(TQuery)
  private
    fMyHandle:TBkgHandle;
    fActiveThread:boolean;
    fThread: TThread;
    fSession:TSession;
    fDatabase:TDatabase;
    fDBParams:TStringList;
    fDBAlias:string;
    fDBDriver:string;
    fDBDatabaseName:string;
    fOnAfterExecute:TNotifyEvent;
    fOnExecute:TNotifyEvent;
    procedure SetfOnExecute(P:TNotifyEvent);
    procedure SetfDBParams(P:TStringList);
 { Private declarations }
  protected
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy; override;
    { Public declarations }
  published
    property ActiveThread : boolean read fActiveThread;
    property Database_AliasName:string read fDBAlias write fDBAlias;
    property Database_Databasename:string read fDBdatabaseName write fDBDatabaseName;
    property Database_DriverName:string read fDBDriver write fDBDriver;
    property Database_Params:TStringList read fDBParams write SetfDBParams;
    property OnAfterExecute: TNotifyEvent read fOnAfterExecute write fOnAfterExecute;
    property OnExecute: TNotifyEvent read fOnExecute write SetfOnExecute;

    function  Database:TDatabase;
    function  Session:TSession;
    procedure Execute;
    procedure BkgTerminate;
    { Published declarations }
  end;

  TBkgThread = class(TThread)
    private
    fBkgQuery    : TBkgQuery;
    { Private declarations }
  protected
    procedure Execute; override;
    { Protected declarations }
  public
      { Public declarations }
  published
    { Published declarations }
  end;


procedure Register;

implementation

constructor TBkgQuery.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fDBParams:=TStringList.Create;
  fMyHandle:=nil;
  fSession:=nil;
  fDatabase:=nil;
  fThread:=nil;
end;

Destructor TBkgQuery.Destroy;
begin
  fDBParams.Free;
  if fSession<>nil then begin fSession.Close; fSession.Free; end;
  if fDatabase<>nil then fDatabase.Free;
  if fThread<>nil then begin
    fThread.Terminate;fThread.WaitFor;
  end;
  inherited Destroy;
end;

procedure TBkgQuery.SetfDBParams(P:TStringList);
begin
  fDBParams.text:=P.text;
end;

function TBkgQuery.Database:TDatabase;
begin
  result:=fDatabase;
end;

function TBkgQuery.Session:TSession;
begin
  result:=fSession;
end;

procedure TBkgQuery.BkgTerminate;
begin
  fThread.Terminate;
  fActiveThread:=false;
end;

procedure TBkgQuery.Execute;
var X:TWinControl;
begin
  if fActiveThread then begin ShowMessage(Name+' Thread is already active'); exit;end;
  if (fMyHandle=nil) then begin
    if Owner<>nil then begin
      if Owner is TForm then X:=TWinControl(Owner);
    end else X:=TWinControl(Screen.ActiveForm);
    fMyHandle:=TBkgHandle.Create(Owner);
    fMyHandle.Width:=0; fMyHandle.Height:=0;
    fMyHandle.Name:=Name+'_Handle';
    fMyHandle.Caption:='';
    fMyHandle.Parent:=X;
    fMyHandle.fMaster:=self;
  end;
  fActiveThread:=true;
  fThread:=TBkgThread.Create(false);
  TBkgThread(fThread).fBkgQuery:=self;
end;

procedure TBkgQuery.SetfOnExecute(P:TNotifyEvent);
begin
  if fActiveThread then Showmessage(Name+' thread is active')
  else fOnExecute:=P;
end;

procedure TBkgThread.Execute;
begin
    with fBkgQuery do begin
      fActiveThread:=true;
      if fSession=nil then begin
        fSession:=TSession.Create(nil);
        fSession.SessionName :='Session_'+fBkgQuery.Name;
      end;
      if fDatabase=nil then begin
        fDatabase:=TDatabase.Create(nil);
        fDatabase.SessionName := fSession.SessionName;
        fDatabase.LoginPrompt:=false;
        if fDataBase.DriverName<>fDBDriver then fDataBase.DriverName:=fDBDriver;
        if fDatabase.AliasName<>fDBAlias then fDatabase.AliasName:=fDBAlias;
        fDatabase.Params.Text:=fDBParams.Text;
        if fDatabase.databaseName<>fDBDatabaseName then fDatabase.DatabaseName:=fDBDatabaseName;
      end;
      if SessionName<>fDatabase.SessionName then begin close; SessionName := fDatabase.SessionName;end;
      if DatabaseName<>fDatabase.DatabaseName then begin close; DatabaseName := fDatabase.DatabaseName; end;
      if trim(fDBAlias)<>'' then begin  //for xBase - file direct open
        if not fDatabase.Connected then fDatabase.Connected:=true;
      end;
    end;
    if assigned(fBkgQuery.fOnExecute) then fBkgQuery.OnExecute(self);
    PostMessage(fBkgQuery.fMyHandle.Handle,UM_BkgReady,0,0);
    Terminate;
    fBkgQuery.fActiveThread:=false;
end;

procedure TBkgHandle.UM_BkgReady(var AMessage : TMessage);
begin
  if fMaster=nil then exit;
  if fMaster is TBkgQuery then
    with TBkgQuery(fMaster) do begin
      if Assigned(fOnAfterExecute) then fOnAfterExecute(fMaster);
    end;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TBkgQuery]);
end;

initialization
  RegisterClass(TBkgHandle);
end.
